/** \file AudRecordWrap.h
	\brief Declaration of the managed bits that are associated
	with the native IAudRecord interface
	\cond DEV
*/

#ifndef AUDRECORDCLI_AUDRECORDWRAP_H
#define AUDRECORDCLI_AUDRECORDWRAP_H

#pragma once

#include "EndpointWrap.h"
#include "CodecWrap.h"
#include "InterfaceWrap.h"

/** The main namespace that contains all the managed wrappers */
namespace AudRecordCLI
{
	namespace IServ = System::Runtime::InteropServices;

	[IServ::StructLayout(IServ::LayoutKind::Sequential)]
	/** Contains all the recording statistics in a session */
	public ref struct AudRecordStats
    {
		/** Number of samples processed */
		ULONGLONG qwSampleCount;
		/** Number of bytes processed */
		ULONGLONG qwByteCount;
		/** Number of samples dropped */
		ULONGLONG qwDroppedSampleCount;
		/** Count of dropped bytes */
		ULONGLONG qwDroppedByteCount;
		/** The current bitrate of the session */
		DWORD dwCurrentBitrate;
		/** The average bitrate of the session */
		DWORD dwAverageBitrate;
		/** The expected bitrate of the session */
		DWORD dwExpectedBitrate;
		/** The current sample rates in kilohertz */
		DWORD dwCurrentSampleRate;
		/** The average sample rates in kilohertz */
		DWORD dwAverageSampleRate;
		/** The expected sample rates in kilohertz */
		DWORD dwExpectedSampleRate;
    };

	/** The state of the recording stream as returned by Recorder::TogglePause and Recorder::QueryPause */
	public enum class AudPausedState
	{
		/** Stream is temporarily stopped */
		AudPaused,
		/** Stream is running normally */
		AudRecording
	};

	/** The main wrapper around the IAudRecord interface */
	public ref class Recorder
	{
	internal:
		/** The native interface that is wrapped */
		IAudRecord* pNativeRecord;
		/** A managed wrapper resulting from Marshal::GetObjectFromIUnknown() */
		Object^ wrapObj;
		Recorder(IAudRecord* pRecord);
	public:
		void Record(Endpoint^ client);
		void ChangeCodec(CodecConfig^ config);
		AudPausedState TogglePause();
		AudPausedState QueryPause();
		void Stop();
		IntPtr AddFileTarget(String^ fileName);
		IntPtr AddNetworkBroadcastTarget(DWORD% port, DWORD maxConnections, bool wantUrl, String^% url);
		IntPtr AddCustomTarget(ITargetSink^ pTarget);
		void RemoveTarget(IntPtr pCookie);
		AudRecordStats^ GetStatistics();

		static Recorder^ Create(CodecConfig^ initialConfig);
	};
}

/** \endcond */

#endif
