/** \file CodecWrap.cpp
	\brief Implements the functions to do with wrapping codecs
	\cond DEV
*/

#include "stdafx.h"
#include "codecwrap.h"

using namespace System::Runtime::InteropServices;

namespace AudRecordCLI
{
	/** Returns a managed list of codec configuration objects from
		a native Codec pointer.

		Each configuration is marshalled from a native CodecConfiguration to a native
		CodecConfig, with the interface pointers going via Marshal::GetObjectForIUnknown()
		\param pCodec The native codec to interrogate
		\returns The new list of the manang-ised configurations
	*/
	ConfigList^ Codecs::GetConfigListForCodec(::Codec* pCodec)
	{
		DWORD num = pCodec->numConfigs;
		ConfigList^ pConfigList = gcnew ConfigList(num);
		::CodecConfiguration* pConfigIter = pCodec->configs, *pConfigEnd = pConfigIter + num;
		while(pConfigIter != pConfigEnd)
		{
			CodecConfig^ pCodecConf = gcnew CodecConfig();
			pCodecConf->Name = gcnew String(pConfigIter->pName);
			IntPtr interfacePtr = IntPtr(pConfigIter->pConfig);
			pCodecConf->DescInterface = Marshal::GetObjectForIUnknown(interfacePtr);
			pConfigList->Add(pCodecConf);
			++pConfigIter;
		}
		return pConfigList;
	}

	/** \endcond */
	/** Get a list of enumerated codecs from the native GetWMACodecConfigs()

		\returns A list containing Codec wrappers
	*/
	CodecList^ Codecs::Get()
	{
		CodecList^ pCodecList = nullptr;
		WinType<::CodecConfigs*, void> pNativeConfigs(NULL, &FreeWMACodecConfigs);
		HRESULT hr = GetWMACodecConfigs(&pNativeConfigs);
		if(SUCCEEDED(hr))
		{
			DWORD numCodecs = (*pNativeConfigs)->num;
			pCodecList = gcnew CodecList(numCodecs);
			::Codec** pIter = (*pNativeConfigs)->pCodecs, **pEnd = pIter + numCodecs;
			while(pIter != pEnd)
			{
				Codec^ pNewCodec = gcnew Codec();
				ConfigList^ pConfigs = GetConfigListForCodec(*pIter);
				// no point returning it if there are no configurations
				if(pConfigs->Count > 0)
				{
					pNewCodec->Name = gcnew String((*pIter)->name);
					pNewCodec->Configs = pConfigs;
					pCodecList->Add(pNewCodec);
				}
				++pIter;
			}
		}
		return pCodecList;
	}
}
