/** \file CodecWrap.h
	\brief Declarations for the wrappers of the native structures
	to do with Codecs and configurations
	\cond DEV
*/

#ifndef AUDRECORDCLI_CODECWRAP_H
#define AUDRECORDCLI_CODECWRAP_H

#pragma once

namespace AudRecordCLI
{
	/** Wrapper for a codec configuration */
	public ref struct CodecConfig
	{
		/** The description of this configuration. Get is public, set is internal */
		property String^ Name
		{
			String^ get()
			{
				return name;
			}
		internal:
			void set(String^ newName)
			{
				name = newName;
			}
		}
	private:
		/** The backing store of the property */
		String^ name;
	internal:
		/** The managed wrapper of the IWMStreamConfig interface */
		property Object^ DescInterface;
	};

	/** Typedef of a collection of codec configs for simplification purposes */
	typedef List<CodecConfig^> ConfigList;

	/** Represents a native codec, and its list of configurations */
	public ref struct Codec
	{
		/** The name of this codec, get is public, set is internal */
		property String^ Name
		{
			String^ get()
			{
				return name;
			}
		internal:
			void set(String^ newName)
			{
				name = newName;
			}
		}

		/** The list of configuration this codec supports. get is public, set is internal */
		property ConfigList^ Configs
		{
			ConfigList^ get()
			{
				return configs;
			}
		internal:
			void set(ConfigList^ newList)
			{
				configs = newList;
			}
		}
	private:
		/** The backing for the name property */
		String^ name;
		/** Backing for the config properties */
		ConfigList^ configs;
	};

	/** Typedef of a collection of endpoints for simplification purposes */
	typedef List<Codec^> CodecList;

	/** Static class that contains the codec related functionality */
	public value struct Codecs abstract sealed
	{
	private:
		static ConfigList^ GetConfigListForCodec(::Codec* pCodec);
	public:
		static CodecList^ Get();
	};
}

/** \endcond */

#endif
