﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.Collections.Generic;
using System.IO;
using System.Windows.Forms;

namespace AudRecordTestCLI
{
    public struct AddSinkResult
    {
        public string file;
        public uint port;
    }

    public partial class AddSink : Form
    {
        private Random rand;
        private AddSinkResult result;
        // this might not actually be textBox.DefaultBackColor
        private Color originalTextBackColor;
        private Color originalTextForeColor;

        public AddSinkResult RealResult
        {
            get
            {
                return result;
            }
        }

        public AddSink()
        {
            InitializeComponent();
            TextFilePath.Text = TextPortNumber.Text = "";
            rand = new Random();
            result = new AddSinkResult();
            originalTextBackColor = TextFilePath.BackColor;
            originalTextForeColor = TextFilePath.ForeColor;
            // used to indicate errors
            TextFilePath.Tag = null;
            TextPortNumber.Tag = null;
        }

        private void ButtonSelect_Click(object sender, EventArgs e)
        {
            if(DlgSaveFile.InitialDirectory == null)
            {
                DlgSaveFile.InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            }
            if (DlgSaveFile.ShowDialog(this) == DialogResult.OK)
            {
                TextFilePath.Text = DlgSaveFile.FileName;
            }
        }

        private void ButtonRandomPort_Click(object sender, EventArgs e)
        {
            int newPort = rand.Next(1000, 65535);
            TextPortNumber.Text = newPort.ToString();
        }

        private void ToggleCreateButton()
        {
            int fileTextLen = TextFilePath.Text.Length;
            int portTextLen = TextPortNumber.Text.Length;
            // enable the button if one, but not both of the boxes have something in
            bool enableState =
                (
                    ((fileTextLen > 3) || (portTextLen >= 4)) &&
                    !((fileTextLen > 0) && (portTextLen > 0))
                );

            ButtonCreate.Enabled = enableState;
        }

        private void RestoreTextColorIfNecessary(object box)
        {
            TextBox control = (TextBox)box;
            if (control.Tag != null)
            {
                control.BackColor = originalTextBackColor;
                control.ForeColor = originalTextForeColor;
                control.Tag = null;
            }
        }

        private void TextFilePath_TextChanged(object sender, EventArgs e)
        {
            RestoreTextColorIfNecessary(sender);
            ToggleCreateButton();
        }

        private void TextPortNumber_TextChanged(object sender, EventArgs e)
        {
            RestoreTextColorIfNecessary(sender);
            ToggleCreateButton();
        }

        private void TextPortNumber_KeyDown(object sender, KeyEventArgs e)
        {
            Keys keyCode = e.KeyCode;
            // if you're not a number you're not coming in
            if (!
                (
                ((keyCode >= Keys.D0) && (keyCode <= Keys.D9)) ||
                ((keyCode >= Keys.NumPad0) && (keyCode <= Keys.NumPad9))
                )
            )
            {
                e.SuppressKeyPress = true;
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Hide();
        }

        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            result.file = null;
            result.port = 0;
            TextBox invalidBox = null;
            bool success = false;
            if ((TextPortNumber.Text != null) && (TextPortNumber.Text.Length > 0))
            {
                success = UInt32.TryParse(TextPortNumber.Text, out result.port);
                invalidBox = TextPortNumber;
            }
            else
            {
                invalidBox = TextFilePath;
                string inputName = TextFilePath.Text;
                string filename = Path.GetFileName(inputName);
                string filePath = Path.GetDirectoryName(inputName);
                char[] invalidNameChars = Path.GetInvalidFileNameChars();
                char[] invalidPathChars = Path.GetInvalidPathChars();
                if ((filename.IndexOfAny(invalidNameChars) == -1) &&
                    (filePath.IndexOfAny(invalidPathChars) == -1)
                )
                {
                    result.file = TextFilePath.Text;
                    success = true;
                }
            }
            if (success)
            {
                DialogResult = DialogResult.OK;
                Hide();
            }
            else
            {
                invalidBox.BackColor = Color.Red;
                invalidBox.ForeColor = Color.Yellow;
                invalidBox.Tag = 1;
            }
        }
    }
}
