/** \file CTNumLimits.h
	\brief Defines a template for compile time min and max of signed types
	\cond DEV
*/
#ifndef CTNUMLIMITS_AUDRECORD_H
#define CTNUMLIMITS_AUDRECORD_H

#pragma once

#include <climits>

/** Provides a compile time equivalent of std::numeric_limits<Type>::min and max 
	
	The unspecialized template is empty so it will cause compile errors for types
	we're not interested in

	\tparam Type The type to get the min and the max values of 
*/
template<class Type>
struct CTNumLimits
{};

/** Limits for signed char */
template<>
struct CTNumLimits<char>
{
	static const char max = SCHAR_MAX; 
	static const char min = SCHAR_MIN;
};

/** Limits for signed short */
template<>
struct CTNumLimits<short>
{
	static const short max = SHRT_MAX; 
	static const short min = SHRT_MIN;
};

/** Limits for signed long */
template<>
struct CTNumLimits<long>
{
	static const long max = LONG_MAX; 
	static const long min = LONG_MIN;
};

/** Limits for floats */
template<>
struct CTNumLimits<float>
{
	static const float max;
	static const float min;
};

// Why C++ doesn't allow these to be initialised like the rest I'll never know
// the __declspec(selectany) allows the multiple definition to be resolved
// by the linker. Since they'll always be the same, there's no
// harm in letting that happen
__declspec(selectany) const float CTNumLimits<float>::max = FLT_MAX;
__declspec(selectany) const float CTNumLimits<float>::min = FLT_MIN;

/** \endcond */

#endif
