/** \file AudRecord\main.cpp
	\brief Defines the main entry point of the dll and other global functions
	\cond DEV
*/

#include "stdafx.h"
#include "ETWEvents.h"
#include "alloc.h"

/** Entry point of the dll

	Register and unregisters the ETW event provider and, in dynamic CRT varieties
	of compilation, disables the unwanted thread notifications.
*/
BOOL APIENTRY DllMain(HMODULE hMod, DWORD dwReason, LPVOID /*lpStaticLoad*/)
{
	BOOL bRet = TRUE;
	switch(dwReason)
	{
		case DLL_PROCESS_ATTACH:
		{
			RegisterEventProvider();
			bRet = InitialiseAlloc(hMod);
#if _DLL
			DisableThreadLibraryCalls(hMod);
#endif
		}
		break;
		case DLL_PROCESS_DETACH:
		{
			UnregisterEventProvider();
			UninitialiseAlloc(hMod);
		}
		break;
	}
	return bRet;
}

/** \endcond */

/** \mainpage AudRecordLib

	\section intro Intro
	AudRecordLib is a audio recording and capture library for Windows Vista and up.
	It takes audio samples from either an output device, like speakers, or an input
	device such as a microphone. These samples are passed them to the Windows Media
	runtime to convert them into either WMA files, network streams, or for applications
	to do as they wish with the data.

	Also in the package is AudRecordCLI, a C++/CLI managed wrapper around everything 
	exposed by the native dll and certain parts of the WMSDK that can be used
	with it.

	\if DEV
	This documentation set contains descriptions of every part of the source code
	for AudRecordLib and AudRecordCLI. To create documentation containing only
	client facing, public portions of the documentation, remove DEV from the
	ENABLED_SECTIONS part of the doxyfile
	\endif

	\section usage Usage
	The simplest usage is to call GetWMACodecConfigs() for a list of installed 
	Windows Media codecs and their CBR and VBR configurations. 
	Use CreateAudRecordInterface() with the desired configuration to get an
	IAudRecord interface pointer and set a target file with IAudRecord::AddFileTarget().
	Set the source of the audio by getting a list of endpoints from GetAudioEndpoints()
	and calling IAudRecord::Record() with the desired audio device.

	All of this is demonstrated in the <a href="..\AudRecordSimple\main.cpp">AudRecordSimple sample code</a>

	\section down Download
	You can download a package containing all the code and binaries from \b
	<a href="http://www.airesoft.co.uk/files/audrecord/audrecord.zip">here</a>

	\section lic License
	All the code is licensed under the MIT license. A copy of this is included in the download.

	\section example Examples
	AudRecordLib comes with three example applications. AudRecordTest is a C++
	application which uses the native dll directly, while AudRecordTestCLI
	is a C# application which consumes the C++/CLI wrapper. The last, AudRecordSimple 
	is a console application which shows the bare minimum required to record
	audio output to a file.

	\section reqs Requirements
	To build AudRecordLib you need a <a href="http://www.microsoft.com/download/en/details.aspx?id=8279">Windows SDK</a>
	capable of targeting Windows Vista, a compiler capable of understanding 
	<a href="http://msdn.microsoft.com/en-us/library/kcwz153a(v=VS.80).aspx">Visual Studio's SSE intrinsics</a>
	and a version of <a href="http://www.boost.org">Boost</a> including boost::function and boost::bind
	(which is practically all of them)
*/
