/** \file messages.h
	\brief Declares the bits required for communicating with the 
	sampling thread
	\cond DEV
*/

#ifndef AUDRECORD_MESSAGES_H
#define AUDRECORD_MESSAGES_H

#pragma once

/** Messages accepted by the sampling thread start at WM_USER 
	Includes start and end sentinels
*/
enum ThreadMessages
{
	FIRST_THREAD_MESSAGE = WM_USER,
	MSG_CREATE_AUDIO_CLIENT = FIRST_THREAD_MESSAGE,
	MSG_RELEASE_AUDIO_CLIENT,
	LAST_THREAD_MESSAGE
};

/** Params for the MSG_CREATE_AUDIO_CLIENT message

	The audio client must be released on the same thread as the one that created it
	so it is created on the sampling thread using this structure
*/
struct CreateAudioClientParams
{
	/** In parameter - the requested size of the audio buffer
	in the created audio client in 100ns units */
	ULONGLONG bufferDuration;
	/** In parameter - specifies the requested size of the audio buffer
	in the created audio client */
	struct IMMDevice* pDevice;

	/** Out parameter - created audio client interface pointer */
	struct IAudioClient* pClient;
	/** Out parameter - interface pointer that captures the audio */
	struct IAudioCaptureClient* pCaptureClient;
	/** Out parameter - wave format of the audio client, must be freed by CoTaskMemFree */
	struct tWAVEFORMATEX* pwfx;
	/** Out parameter - result value */
	HRESULT hr;
};

/** Params for the MSG_RELEASE_AUDIO_CLIENT message

	The structure is passed to sampling thread to release the interfaces
	which must be released on the same thread which created them.
*/
struct ReleaseAudioClientParams
{
	/** In parameter - audio client interface pointer returned in CreateAudioClientParams */
	struct IAudioClient* pClient;
	/** In parameter - previously acquired capture client*/
	struct IAudioCaptureClient* pCaptureClient;
};

// documentation of these is in messages.cpp
void ActOnMessage(const MSG& msg);
BOOL SendThreadMessage(DWORD threadId, UINT msg, PVOID pParam);

/** \endcond */
#endif
