/** \file nullsink.h
	\brief Declares the NullSink class
	\cond DEV
*/

#ifndef NULLSINK_AUDRECORD_H
#define NULLSINK_AUDRECORD_H

#pragma once

#include <wmsdk.h>
#include <vector>

/** A dummy sink class that can be added to a IWMWriter session
	mainly to test that the writer can write. 
	
	Nothing is done with the data it is given. It is designed to
	be allocated on the stack, AddRef and Release do nothing.
*/
class NullSink : public IWMWriterSink
{
	/** Whether the stream has real time samples */
	BOOL realTime;
public:
	NullSink(BOOL realTime);
	// IUnknown
	STDMETHODIMP QueryInterface(REFIID iid, LPVOID* ppv);
	STDMETHODIMP_(ULONG) AddRef();
	STDMETHODIMP_(ULONG) Release();

	// IWMWriterSink
	STDMETHODIMP OnHeader(INSSBuffer* pHeader);
	STDMETHODIMP IsRealTime(BOOL* pRealTime);
	STDMETHODIMP AllocateDataUnit(DWORD dataSize, INSSBuffer** ppBuffer);
	STDMETHODIMP OnDataUnit(INSSBuffer* pData);
	STDMETHODIMP OnEndWriting();
};

/** \endcond */

#endif
